# Screenshot Tests

Prevent visual regressions by running screenshot tests on every PR.

## Quick start

### API credentials

Add the following to your `~/.bash_profile` or `~/.bashrc` file:

```bash
export CBT_USERNAME='you@example.com'
export CBT_AUTHKEY='example'
export MDC_GCLOUD_SERVICE_ACCOUNT_KEY_FILE_PATH='path/to/gcp-credentials.json'
```

Credentials can be found here:

* [CrossBrowserTesting.com > Account](https://crossbrowsertesting.com/account) \
    `Authkey` is listed under the `User Profile` section
* [Google Cloud Console > IAM & admin > Service accounts](https://console.cloud.google.com/iam-admin/serviceaccounts?project=material-components-web) \
    Click the `︙` icon on the right side of the service account, then choose `Create key` 

### Test your changes

Modify a Sass file:

```bash
echo '.mdc-button:not(:disabled){color:red}' >> packages/mdc-button/mdc-button.scss
```
 
Run the tests:

```bash
npm run screenshot:test
```

You should see something like this in the terminal:

```
Found 65 screenshot diffs!
https://storage.googleapis.com/mdc-web-screenshot-tests/advorak/2018/05/22/16_23_58_618/af3d7271f/report.html
```

## Basic usage

### Updating "golden" screenshots

```bash
npm run screenshot:update-goldens
```

In addition to running the tests, this command will also update your local `test/screenshot/golden.json` file with the 
newly captured screenshots.

### Rerunning a subset of tests

You can rerun a subset of the tests without running the entire suite, filtering by browser and/or URL:

```bash
npm run screenshot:update-goldens -- \
  --mdc-include-url=mdc-button/classes/dense \
  --mdc-include-browser=ie@11
```

**IMPORTANT:** Note the `--` between the script name and its arguments. This is required by `npm`.

You can rerun multiple screenshots by passing an argument multiple times:

```bash
npm run screenshot:update-goldens -- \
  --mdc-include-url=mdc-button/classes/dense \
  --mdc-include-url=mdc-fab/classes/mini \
  --mdc-include-browser=ie@11 \
  --mdc-include-browser=chrome
```

You can also _exclude_ specific browsers and URLs:

```bash
npm run screenshot:update-goldens -- \
  --mdc-exclude-url=mdc-button \
  --mdc-exclude-browser=edge
```

These flags are treated as regular expressions, so partial matches are possible. For example:

* `ie@11` matches `desktop_windows_ie@11`
* `chrome` matches `desktop_windows_chrome@latest` and `mobile_android_chrome@latest`

See `test/screenshot/browser.json` for the full list of supported browsers.

### Local dev server

The deprecated `npm run dev` command has been replaced by:

```bash
npm start
```

Open http://localhost:8080/ in your browser to view the test pages.

Source files are automatically recompiled when they change.

## Advanced usage

To see all available CLI flags, run:

```bash
npm run screenshot:test -- --help
```

**IMPORTANT:** Note the `--` between the script name and its arguments. This is required by `npm`.

The same set of flags work in both `screenshot:test` and `screenshot:update-goldens`.

### Public demos

```bash
npm run screenshot:upload-assets
```

This will upload all test assets (HTML/CSS/JS files) to a public URL and print the URL to the terminal.

The URL can then be shared with designers or other developers.

### Diffing against a local `golden.json` file

By default, screenshots are diffed against `origin/master:test/screenshot/golden.json`.

To diff against a local `golden.json` file, run:

```bash
npm run screenshot:test -- --mdc-diff-base=test/screenshot/golden.json
```

URLs are also supported:

```bash
npm run screenshot:test -- --mdc-diff-base=https://storage.googleapis.com/mdc-web-screenshot-tests/advorak/2018/05/22/17_34_19_887/c8c29033e/golden.json
```

### Diffing against another branch

By default, screenshots are diffed against `origin/master:test/screenshot/golden.json`.

To diff against a different branch, run:

```bash
npm run screenshot:test -- --mdc-diff-base=fix/fab/icon-alignment-ie11
```

## Writing tests

**Goal**: Cover of every line of CSS generated by our Sass. This includes all CSS selectors.

### Guidelines

1.  Keep HTML files _small_ and _focused_. This makes it easier to review diffs.

2.  Each page should target a single logical "variant" of a component. For example:
    - One "block" CSS class (e.g., `mdc-button`)
    - One "modifier" CSS class (e.g., `mdc-button--dense`)
    - One Sass mixin (e.g., `mdc-button-ink-color`)

3.  File structure:

        /test/screenshot
            /mdc-foo
                /classes
                    - baseline.html
                    - dense.html
                    - ...
                /mixins
                    - custom.scss
                    - fill-color.html
                    - filled-accessible.html
                    - ink-color.html
                    - ...

4.  Do not test _combinations_ of mixins and modifier classes unless:
    1. It's necessary to prevent a regression; or
    2. We explicitly support the combination, and the implementation is likely to have bugs (e.g., `mdc-button--dense` and `mdc-button-outline-width` both set `line-height`)

### Example test page

![filled button screenshot](https://user-images.githubusercontent.com/409245/40395978-7f3e0476-5ddf-11e8-9262-eb0dfacb05e5.png)
